 /*----------------------------------------------------------------*
  *    Name: panels.sas                                            *
  *   Title: Macro to display a set of plots in rectangular panels *
  *           using PROC GREPLAY.                                  *
  *     Doc: http://www.math.yorku.ca/SCS/vcd/panels.html          *
  *----------------------------------------------------------------*
  *  Author:  Michael Friendly            <friendly@yorku.ca>      *         
  * Created:  1 Mar 1994 13:16:36                                  *         
  * Revised:  19 Nov 1998 10:24:31                                 *         
  * Version:  1.5                                                  *
  *  - EQUATE default changed to Y                                 *         
  *----------------------------------------------------------------*/

/*=
=Description:

 The PANELS macro constructs a template in which to replay a series of
 graphs, assumed all the same size, in a rectangular array of R
 rows and C columns. By default, the panels are displayed
 left-to-right across rows,  starting either from the top
 (ORDER=DOWN) or bottom (ORDER=UP).  If the number of rows and
 columns are unequal, the aspect ratio of individual panels can
 be maintained by setting equate=Y.  It is assumed that all the
 plots have already been created, and stored in a graphics catalog
 (the default, WORK.GSEG is used automatically by SAS/GRAPH
 procedures).

 For interactive use within the SAS Session Manager you should be
 aware that all plots are stored cumulatively in the graphics
 catalog throughout your session, unless explicitly changed with
 the GOUT= option in graphics procedures or macros  To create multiple
 panelled plots you can use the FIRST= and LAST= parameters or a REPLAY=
 list to specify which plots are used in a given call.
   
=Usage:

 Call the PANELS macro after the steps which create the graphs in the
 graphics catalog.  The GDISPLA macro may be used to suppress the
 display of the original full-sized graphs.
 The ROWS= and COLS= parameters must be specified.
 
   goptions hsize=7in vsize=5in;
	%gdispla(OFF);
	proc gplot data=mydata;
		plot y * x = group;
		by sex;
	
	%gdispla(ON);
	%panels(rows=1, cols=2);
	
==Parameters:
   
* ROWS=
* COLS=
   The ROWS= and COLS= arguments are required, and specify the
   size of the array of plots to be displayed. These are the only
   required arguments.
   
* PLOTS=
   If there are fewer than &ROWS*&COLS plots, specify the number
   as the PLOTS= argument. Optionally, there can be an additional
   plot, which is displayed (as a GSLIDE title, for example) in
   the top nn% of the display, as specified by the TOP= argument.
   
* TOP=
   If TOP=nn is specified, the top nn% of the display is reserved
	for one additional panel (of width 100%), to serve as the plot
	title or annotation.

* ORDER=
   The ORDER= argument specifies the order of the panels in the
   display. ORDER=UP means that the panels in the bottom row are
	are drawn first, and  numbered 1, 2, ..., &COLs. ORDER=DOWN means
	that the panels in the top row are drawn first, numbered 1, 2, ..., &COLs.
   
* EQUATE=
	The EQUATE= argument determines if the size of the panels is adjusted
	so that the aspect ratio of the plots is preserved.  If EQUATE=Y,
	the size of each plot is adjusted to the maximum of &ROWS and &COLS.
	This is usually desired, as long as the graphic options HSIZE and
	VSIZE are the same when the plots are replayed in the panels
	template as when they were originally generated.  The default is
	EQUATE=Y.
	
* REPLAY=
   The REPLAY= argument specifies the list of plots to be replayed
   in the constructed template, in one of the forms used with the
   PROC GREPLAY REPLAY statement, for example, REPLAY=1:1 2:3 3:2
   4:4 or REPLAY=1:plot1 2:plot3 3:plot2 4:plot4
   
* TEMPLATE=
   The name of the template constructed to display the plots. The
	default is TEMPLATE=PANEL&ROWS.&COLS.

* TC=
   The name of the template catalog used to store the template.
	You may use a two-part SAS data set name to save the template
	permanently.

* FIRST=
* LAST=
   By default, the REPLAY= argument is constructed to replay plot
   i in panel i.  If the REPLAY= argument is not specified, you can
   override this default assignment by specifying FIRST= the sequential
   number of the first graph in the graphics catalog to plot (default:
   first=1), where:                       
		>0 means absolute number of first graph,          
		<1 means number of first graph relative to last (i.e. 0 means 
		 last graph only, -1 means first is one before last, etc.) 
		                            
   last=0          Number of last graph to plot                        
		>0 means absolute number of last graph,           
		<1 means number of last graph relative to number  
		of graphs in the catalog (i.e. 0 means last graph 
		in the catalog, -1 means one before last, etc.)

* GIN=
	GIN specifies the name of the input graphics catalog, from which the
	plots to be replayed are taken. The default is GIN=WORK.GSEG.

* GOUT=
	GOUT= specifies the name of the graphics catalog in which the panelled
	plot is stored. The default is GOUT=WORK.GSEG.
 =*/

%macro panels(
	rows=, 				    /* number of rows of plots                */
	cols=, 				    /* number of columns of plots             */ 
	plots=&rows * &cols,  /* total number of plots                  */
	top=,					    /* percent of display for top panel       */ 
	order=UP,             /* start at bottom (UP) or top (DOWN)?    */
	replay=,				    /* List of plots to replay                */
	equate=Y,			    /* Adjust sizes to maintain aspect ratio? */ 
	template=panel&rows.&cols,   /* name of template                */
	tc=panels,			    /* name of template catalog					 */
	first=1,              /* number of the first catalog entry used */
	last=0,               /* number of the last catalog entry used  */
	gin=gseg,
	gout=gseg);

%local i j panl panl1 lx ly tx ty sx sy;
%*put PANELS: plots= &plots;

%let order=%upcase(&order);
%let equate=%substr(%upcase(&equate),1,1);

%let abort=0;
%let showgout=0;
%if &rows=%str() | &cols=%str()
	%then %do;
		%put ERROR: The ROWS= and COLS= parameters must be specified;
      %let abort=1;
		%goto DONE;
	%end;
	
  /* determine how many graphs are in the catalog */
%let npics=0;
  proc catalog catalog=&gin et=grseg;
  	contents out=_cont_;
	run;
%if &syserr > 4 %then %let abort=1; %if &abort %then %goto DONE;
  data _null_; 
   set _cont_ end=last; 
	if (last) then call symput('npics',left(put(_n_,5.))); 
	run;
%put NOTE: The graphics catalog &gin contains &npics graphic entries;
%if &npics=0 %then %goto DONE;
%if &syserr > 4 %then %let abort=1; %if &abort %then %goto DONE;

  %if (&last<1) %then %do;
    %if (%eval(&npics+&last)>=1) %then %let last = %eval(&npics + &last);
    %else %do;
      %put NOTE (panels): last=&last too low - setting last=1.;
      %let last = 1;
    %end;
  %end;
  %else %if (&last>&npics) %then %do;
    %put NOTE (panels): last=&last too high - setting last=&npics.;
    %let last = &npics;
  %end;

  %if (&first>&last) %then %do;
  	%put NOTE (panels): first=&first too high - no plots done.;
	%goto done;
	%end;

  %if (&first<1) %then %do;
      %if (%eval(&last+&first)>=1) %then %let first = %eval(&last + &first);
      %else %do;
        %put NOTE (panels): first=&first too low - setting first=1.;
        %let first = 1;
		  %let showgout=1;
      %end;
  %end;

    %put NOTE (panels): plotting &first to &last ....;
  

%*-- If the replay list is not given, construct it, ordering the
	plots across each row, but starting at either the bottom row
	(order=UP) or the top row (order=DOWN);
	
%if &replay=%str() %then %do; 
	%do i = 1 %to &plots;
	      %let j = %eval(&first+&i-1);
			%let replay = &replay &i:&j ;
		%end;
	%if &top ^= %str() 
		%then %let replay = &replay %eval(&plots+1):%eval(&first+&plots+1);
	%end;
	
%*-- Calculate panel size and starting location;
data _null_;
   %if &top = %str()
       %then %do;  ty=100;      %end;
       %else %do;  ty=100-&top; %end;
   tx=100;
	%if &equate=N %then %do;
		hsize = tx/&cols;
		vsize = ty/&rows;
		sx = 0; sy = 0;
	%end;
	%else %do;
		np = max(&rows,&cols);
		hsize = round(tx/np,.01);
		vsize = round(ty/np,.01);
		sx = round((np - &cols) * hsize/2,.01);
		sy = round((np - &rows) * vsize/2,.01);
	%end;
	lx = round(hsize + sx,.01);
	ly = round(vsize + sy,.01);
*	put hsize= vsize= tx= ty= sx= sy= lx= ly=;

	%if &order=DOWN %then %do;
		ly = round(ty - sy,.01);
		sy = ly - vsize;
		vsize = -vsize;
	%end;			

	put 'PANELS: ' hsize= vsize= tx= ty= sx= sy= lx= ly=;
   call symput('hsize', put(hsize,6.2));
   call symput('vsize', put(vsize,6.2));
   call symput('lx', put(lx,6.2));
   call symput('ly', put(ly,6.2));
   call symput('tx', put(tx,6.2));
   call symput('ty', put(ty,6.2));
   call symput('sx', put(sx,6.2));
   call symput('sy', put(sy,6.2));
run;

proc greplay igout=&gin
              gout=&gout  nofs
             template=&template
             tc=&tc ;

%* ---------------------------------------------------------------;
%* Generate a TDEF statement for a plot matrix                    ;
%* Start with (1,1) panel in lower left, and copy it across & up  ;
%* ---------------------------------------------------------------;
   TDEF &template DES="panels template &rows x &cols"
   %let panl=0;
   %* let lx = &hsize;
   %* let ly = &vsize;
   %do i = 1 %to &rows;
   %do j = 1 %to &cols;
		 %if &panl > %eval(&plots) %then %goto fini;
       %let panl  = %eval(&panl + 1);
       %if &j=1 %then
          %do;
             %if &i=1 %then %do;      %* (1,1) panel;
               &panl/
                ULX=&sx  ULY=&ly     URX=&lx  URY=&ly
                LLX=&sx  LLY=&sy     LRX=&lx  LRY=&sy
                %end;
             %else
                %do;                  %* (i,1) panel;
                   %let panl1 = %eval(&panl - &cols );
               &panl/ copy= &panl1 xlatey= &vsize
                %end;
          %end;
       %else
          %do;
               %let panl1 = %eval(&panl - 1);
               &panl/ copy= &panl1 xlatex= &hsize
          %end;
   %end;
   %end;
%fini:
   %if &top ^= %str() %then %do;
       %let panl = %eval(&panl + 1);
         &panl/
          ULX=0  ULY=100     URX=&tx URY=100
          LLX=0  LLY=&ty     LRX=&tx LRY=&ty
   %end;
     %str(;);      %* end the TDEF statement;
   %if &replay ^= %str() %then %do;
      TREPLAY &replay;
		LIST template;
   %end;
	%if &showgout %then %str(LIST IGOUT;);
run;	quit;

%DONE:
%if &abort %then %put ERROR: The PANELS macro ended abnormally.;
options notes;
%mend;
