 /*-----------------------------------------------------------------*
  |     Name: table.sas                                             |
  |    Title: Construct a grouped frequency table, with recoding    |
  |     Doc: http://www.math.yorku.ca/SCS/vcd/table.html            |
  | ----------------------------------------------------------------|
  |    Procs: freq printto                                          |
  |  Macdefs: table join                                            |
  | Requires: %tempfile %tempdel                                    |
  | ----------------------------------------------------------------|
  |   Author: Michael Friendly               <friendly@yorku.ca>    |
  |  Created: 09 Jul 1999 16:52:05                                  |
  |  Revised: 11 Jul 1999 11:16:11                                  |
  |  Version: 1.0                                                   |
  *-----------------------------------------------------------------*/
/*=
=Description:

 The TABLE macro constructs a grouped frequency table suitable for 
 input to %mosaics.  The input data may be individual observations,
 or a contingency table,  which may be collapsed to fewer variables.
 Factor variables may be converted to character using user-supplied
 formats.

=Usage:

 The TABLE macro takes 7 keyword arguments.  The VAR= parameter
 is required.
 
==Parameters:

* DATA=              The name of the input dataset.  [Default: DATA=_LAST_]

* VAR=               Names of all factor (classification) variables to be
                     included in the
                     output dataset.  The observations are summed over
                     any other factors, weighted by the WEIGHT= variable,
                     if any.

* CHAR=              If non-blank, forces the VAR= variables to be
                     converted to character variables (using formatted
                     values) in the output dataset.  If CHAR= a
                     numeric value (e.g., CHAR=8), it specifies the
                     length of each character variable; otherwise, the
                     character variables default to length 16.

* WEIGHT=            Name of a frequency variable, if the input dataset
                     is already in frequency form.

* ORDER=             Specifies the order of the variable levels used
                     in the PROC FREQ step.  ORDER=INTERNAL|FREQ|DATA|
                     FORMATTED are valid option values.

* FORMAT=            List of variable(s), format pairs (suitable for a
                     format statement).  The FORMAT= option may be used
                     to recode the values of any of the VAR= variables.

* OUT=               Name of output dataset.  The variables in the output
                     dataset are the VAR= variables, plus COUNT, the
                     frequency variable for each cell. [Default: OUT=TABLE]

=Limitations:

None of the factor variables may be named COUNT.

=Example:

 This example reads a three-way frequency table (gender x admit x dept),
 where admit and dept are numeric variables, and collapses it to a
 two-way table, with Gender and Admit as character variables.
 
   %include data(berkeley);
   %table(data=berkeley, var=gender admit, weight=freq, char=Y,
          format=admit admit. gender $sex., order=data, out=berk2);
   %mosaic(data=berk2, var=Gender Admit);

 The formats admit. and $sex. are created with proc format:

   proc format;
      value admit 1="Admitted" 0="Rejected";
      value $sex  'M'='Male'   'F'='Female';

=*/ 

%macro table (
   data=_last_,     /* Name of input dataset                        */
   var=,            /* Names of all factor variables                */
   char=,           /* Force factor variables to character?         */
   weight=,         /* Name of a frequency variable                 */
   order=,          /* Specifies the order of the variable levels   */
   format=,         /* List of var, format pairs                    */
   out=table        /* Name of output dataset                       */
   );

%let abort=0;
%let ls=120;
%*-- Save original linesize;
%if &sysver>6.10 %then %do;
   %let lso=%sysfunc(getoption(ls,keyword));
   %let pso=%sysfunc(getoption(ps,keyword));
   %let dto=%sysfunc(getoption(date));
   %let cto=%sysfunc(getoption(center));
   %end;
%else %do; %let lso=; %let pso=; %let dto=; %let cto=; %end;


%if %length(&var)=0 %then %do;
   %put ERROR:  The VAR= variables must be specified.;
   %end;

%let table = %join(&var, *);

proc freq data=&data
   %if %length(&order) %then order=&order;
   ;
   %if %length(&weight) %then %do;
      weight &weight;
      %end;
   tables &table / noprint sparse out=&out(drop=percent);
   %if %length(&format) %then %do;
      format &format;
      %end;

%if %length(&char)>0 %then %do;
   /*
    * Force the VAR= variables to character.  To do this cleanly, we
    * resort to printing the &out dataset, then reading it back as
    * character.  
    */
   %tempfile(table,&ls);
   proc printto new print=table;
   options nodate nocenter nonumber ls=&ls;
   proc print data=&out;
      id &var;
      var count;
      run;
   %if &syserr > 4 %then %let abort=1; %if &abort %then %goto DONE;
   proc printto print=print;


   %let tvar = %join(&var, $) $;
   %*put tvar=&tvar;

   %if %verify(&char,  %str(0123456789))=0
      %then %let clen=&char;
      %else %let clen=16;
   data &out;
      infile table length=len;
      length string $&ls &var $&clen;
      retain skipping 1;
      drop string skipping;
      input @1 string $varying. len @;
      if skipping=0 & string ^= ' ' then do;
         input @1 &tvar count;
         output;
         end;
      else input;
      if index(string,'COUNT')>0 then skipping=0;
   run;
   *proc contents data=&out;

%tempdel(table);
%end;

%done:;
%if &abort %then %put ERROR: The TABLE macro ended abnormally.;
options notes &lso &pso &dto &cto;
%if &sysver<=6.10 %then %do;
	options center date ls=80;
	%end;

%mend;

 /*=
=Description:

Join the &delim-separated words in &string with &sep.

=Usage:

 %let result = %join(A B C, *);   *-- returns: A * B * C;

==Parameters:

* STRING        A &delim-separated string of 'words'    

* SEP           The separator character(s) used between each pair

* DELIM         The delimiters [Default: %str( )]

 =*/

%macro join(string, sep, delim);

   %local count word;
   %if %length(&delim) %then %let delim=%str( );
   %let count=1;
   %let word = %scan(&string,&count,%str( ));
   %let result = &word;
   %do %while(&word^= );
       %let count = %eval(&count+1);
       %let word = %scan(&string,&count,%str( ));
       %if %length(&word)
          %then %let result = &result &sep &word;
   %end;
   &result
   
%mend;

/*
%let v=A ;
%let j=%join(&v, *);
%put joined: &j;

%let v=A B C;
%let j=%join(&v, *);
%put joined: &j;
*/
